@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import datetime
 import itertools
 import json
 import logging
@@ -17,6 +18,7 @@ import sys
 import uuid
 
 import mock
+from oslo_utils import encodeutils
 import requests
 import requests.auth
 import six
@@ -987,7 +989,12 @@ class SessionAuthTests(utils.TestCase):
                 'X-OpenStack-Request-ID': request_id,
             })
 
-        resp = sess.get(self.TEST_URL)
+        resp = sess.get(
+            self.TEST_URL,
+            headers={
+                encodeutils.safe_encode('x-bytes-header'):
+                encodeutils.safe_encode('bytes-value')
+            })
 
         self.assertEqual(response, resp.json())
 
@@ -998,6 +1005,7 @@ class SessionAuthTests(utils.TestCase):
 
         self.assertIn('curl -g -i -X GET {url}'.format(url=self.TEST_URL),
                       request_output)
+        self.assertIn('-H "x-bytes-header: bytes-value"', request_output)
         self.assertEqual('[200] Content-Type: application/json '
                          'X-OpenStack-Request-ID: '
                          '{id}'.format(id=request_id), response_output)
@@ -1010,6 +1018,26 @@ class SessionAuthTests(utils.TestCase):
                 key=response_key, val=response_val),
             body_output)
 
+    def test_collect_timing(self):
+        auth = AuthPlugin()
+        sess = client_session.Session(auth=auth, collect_timing=True)
+        response = {uuid.uuid4().hex: uuid.uuid4().hex}
+
+        self.stub_url('GET',
+                      json=response,
+                      headers={'Content-Type': 'application/json'})
+
+        resp = sess.get(self.TEST_URL)
+
+        self.assertEqual(response, resp.json())
+        timings = sess.get_timings()
+        self.assertEqual(timings[0].method, 'GET')
+        self.assertEqual(timings[0].url, self.TEST_URL)
+        self.assertIsInstance(timings[0].elapsed, datetime.timedelta)
+        sess.reset_timings()
+        timings = sess.get_timings()
+        self.assertEqual(len(timings), 0)
+
 
 class AdapterTest(utils.TestCase):
 
